@@ -10,12 +10,10 @@ package Data::Fetch;
 #	must apply in writing for a licence for use from Nigel Horne at the
 #	above e-mail.
 
-# use 5.12.0;	# Threads before that are apparently not good
-use 5.10.0;	# Earliest version that Coro works with
+use 5.12.0;	# Threads before that are apparently not good
 use strict;
 use warnings;
-use Coro;
-# use threads;
+use threads;
 
 =head1 NAME
 
@@ -23,11 +21,11 @@ Data::Fetch - give advance warning that you'll be needing a value
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 =head1 SYNOPSIS
 
@@ -56,7 +54,7 @@ sub new {
 
 	return unless(defined($class));
 
-	return bless { }, $class;
+	return bless({ lock => 0}, $class);
 }
 
 =head2 prime
@@ -85,26 +83,28 @@ sub prime {
 	}
 
 	if($self->{values} && $self->{values}->{$object} && $self->{values}->{$object}->{status}) {
-		return $self;
+		my @call_details = caller(0);
+		die 'Attempt to prime twice at ', $call_details[2], ' of ', $call_details[1];
 	}
-	# $self->{values}->{$object}->{thread} = threads->create(sub {
-		# my $o = shift;
-		# my $m = shift;
-		# if(my $a = shift) {
-			# return eval '$o->$m($a)';
-		# }
-		# return eval '$o->$m()';
-	# }, $args{'object'}, $args{'message'}, $args{'arg'});
 
 	$self->{values}->{$object}->{status} = 'running';
-	$self->{values}->{$object}->{thread} = async {
-		my $o = $args{object};
-		my $m = $args{message};
-		if(my $a = $args{arg}) {
+
+	$self->{values}->{$object}->{thread} = threads->create(sub {
+		my ($o, $m, $a) = @_;
+		if($a) {
 			return eval '$o->$m($a)';
 		}
 		return eval '$o->$m()';
-	};
+	}, $args{object}, $args{message}, $args{arg});
+
+	# $self->{values}->{$object}->{thread} = async {
+		# my $o = $args{object};
+		# my $m = $args{message};
+		# if(my $a = $args{arg}) {
+			# return eval '$o->$m($a)';
+		# }
+		# return eval '$o->$m()';
+	# };
 
 	return $self;	# Easily prime lots of values in one call
 }
@@ -120,6 +120,9 @@ Takes one optional parameter:
 
     arg - passes this argument to the message
 
+If you don't prime it will still work and store the value for subsequent calls,
+but in this scenerio you gain nothing over using CHI to cache your values.
+
 =cut
 
 sub get {
@@ -134,8 +137,17 @@ sub get {
 	}
 
 	if(!defined($self->{values}->{$object}->{status})) {
-		my @call_details = caller(0);
-		die "Need to prime before getting at line ", $call_details[2], ' of ', $call_details[1];
+		# my @call_details = caller(0);
+		# die 'Need to prime before getting at line ', $call_details[2], ' of ', $call_details[1];
+		my ($o, $m, $a) = ($args{object}, $args{message}, $args{arg});
+		my $rc;
+		if($a) {
+			$rc = eval '$o->$m($a)';
+		} else {
+			$rc = eval '$o->$m()';
+		}
+		$self->{values}->{$object}->{status} = 'complete';
+		return $self->{values}->{$object}->{value} = $rc;
 	}
 	if($self->{values}->{$object}->{status} eq 'complete') {
 		return $self->{values}->{$object}->{value};
@@ -160,7 +172,9 @@ sub DESTROY {
 
 	foreach my $o(values %{$self->{values}}) {
 		if($o->{thread}) {
-			# $o->{thread}->detach();
+			if($o->{thread}->is_running()) {
+				$o->{thread}->detach();
+			}
 			delete $o->{thread};
 			delete $o->{value};
 		}
@@ -173,6 +187,8 @@ Nigel Horne, C<< <njh at bandsman.co.uk> >>
 
 =head1 BUGS
 
+ARRAY contexts not supported.
+
 Can't pass more than one argument to the message.
 
 I would not advise using this to call messages that change values in the object.
